
package com.example.testapk;

import com.example.testapk.provider.TestDatabaseHelper;
import com.example.testapk.provider.TestDatabaseHelper.ConcreteColumns;
import com.example.testapk.provider.TestDatabaseHelper.MemoryColumns;
import com.example.testapk.provider.TestMemoryProvider;

import android.app.Activity;
import android.app.LoaderManager.LoaderCallbacks;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.AdapterView;
import android.widget.CursorAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;

public class MainActivity extends Activity {

    private static final int LOADER_METADATA = 1;

    private static final int TOKEN_ADD = 0;

    private static final int TOKEN_DELETE = 1;

    private ListView mList;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.aduq_layout);
        mList = (ListView)findViewById(R.id.listView1);
        mList.setOnCreateContextMenuListener(this);
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View view, ContextMenuInfo menuInfo) {
        AdapterView.AdapterContextMenuInfo info;
        try {
            info = (AdapterView.AdapterContextMenuInfo)menuInfo;
        } catch (ClassCastException e) {
            return;
        }

        if (mList == null || info.position > mList.getCount() - mList.getFooterViewsCount() - 1) {
            return;
        }

        Cursor c = (Cursor)mList.getItemAtPosition(info.position);
        String name = c.getString(1);
        String pwd = c.getString(2);
        name = name == null ? "" : name;
        pwd = pwd == null ? "" : pwd;

        menu.add(0, 1, 0, "Delete " + name + "?");

        super.onCreateContextMenu(menu, view, menuInfo);
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo info;
        try {
            info = (AdapterView.AdapterContextMenuInfo)item.getMenuInfo();
        } catch (ClassCastException e) {
            return false;
        }

        if (mList == null || info.position > mList.getCount() - mList.getFooterViewsCount() - 1) {
            return false;
        }

        Cursor c = (Cursor)mList.getItemAtPosition(info.position);
        String name = c.getString(1);
        name = name == null ? "" : name;

        mHandler.removeMessages(TOKEN_DELETE);
        Message msg = mHandler.obtainMessage(TOKEN_DELETE);
        msg.obj = new String[] {
                name
        };
        mHandler.sendMessage(msg);

        return true;
    }

    public void onAddButtonClick(View view) {
        EditText et = (EditText)findViewById(R.id.editText1);
        String user = et.getText().toString();
        et = (EditText)findViewById(R.id.editText2);
        String password = et.getText().toString();

        if(TextUtils.isEmpty(user) || TextUtils.isEmpty(password)) {
            return;
        }

        mHandler.removeMessages(TOKEN_ADD);
        Message msg = mHandler.obtainMessage(TOKEN_ADD);
        msg.obj = new String[] {
                user, password
        };
        mHandler.sendMessage(msg);
    }

    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            if (msg != null) {
                if (msg.what == TOKEN_ADD) {
                    String[] arr = (String[])msg.obj;
                    ContentResolver cr = MainActivity.this.getContentResolver();
                    Uri uri = Uri.withAppendedPath(TestMemoryProvider.CONTENT_URI,
                            TestDatabaseHelper.Tables.CONCRETE);
                    ContentValues values = new ContentValues();
                    values.put(ConcreteColumns.UID, arr[0]);
                    values.put(MemoryColumns.PASSWORD, arr[1]);
                    cr.insert(uri, values);
                } else if(msg.what == TOKEN_DELETE) {
                    String[] arr = (String[])msg.obj;
                    ContentResolver cr = MainActivity.this.getContentResolver();
                    Uri uri = Uri.withAppendedPath(TestMemoryProvider.CONTENT_URI,
                            TestDatabaseHelper.Tables.CONCRETE);
                    cr.delete(uri, ConcreteColumns.UID + "=?", arr);
                }
            }
        }
    };

    public void onQueryButtonClick(View view) {
        getLoaderManager().destroyLoader(LOADER_METADATA);
        getLoaderManager().restartLoader(LOADER_METADATA, null, mDataLoaderCallbacks);
    }

    private LoaderCallbacks<Cursor> mDataLoaderCallbacks = new LoaderCallbacks<Cursor>() {
        @Override
        public CursorLoader onCreateLoader(int id, Bundle bundle) {
            Uri uri = Uri.withAppendedPath(TestMemoryProvider.CONTENT_URI,
                    TestDatabaseHelper.Tables.CONCRETE);
            String[] COLUMNS = new String[] {
                    ConcreteColumns._ID,
                    ConcreteColumns.UID,
                    MemoryColumns.PASSWORD
                };
            return new CursorLoader(MainActivity.this, uri, COLUMNS, null, null, null);
        }

        @Override
        public void onLoadFinished(Loader<Cursor> cursorLoader, Cursor cursor) {
            mList.setAdapter(new SimpleCursorAdapter(MainActivity.this,
                    R.layout.list_item,
                    cursor,
                    new String[]{ConcreteColumns._ID, ConcreteColumns.UID,MemoryColumns.PASSWORD},
                    new int[]{R.id.user_id, R.id.user,R.id.password},
                    CursorAdapter.FLAG_REGISTER_CONTENT_OBSERVER
                    ));
        }

        @Override
        public void onLoaderReset(Loader<Cursor> cursorLoader) {
        }
    };
}
