package com.example.testapk.provider;

import com.example.testapk.provider.TestDatabaseHelper.ConcreteColumns;
import com.example.testapk.provider.TestDatabaseHelper.MemoryColumns;
import com.example.testapk.provider.TestDatabaseHelper.Tables;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;

import java.util.HashMap;

public class TestMemoryProvider extends ContentProvider {

    public final static String AUTHORITY = "com.example.testapk.provider";

    public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY);

    private static final int CONCRETE_TABLE = 0;

    private static final int MEMORY_TABLE = 1;

    private static UriMatcher sUriMatcher;

    private SQLiteOpenHelper mDbHelper;

    static {
        sUriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        sUriMatcher.addURI(AUTHORITY, TestDatabaseHelper.Tables.CONCRETE, CONCRETE_TABLE);
        sUriMatcher.addURI(AUTHORITY, TestDatabaseHelper.Tables.MEMORY, MEMORY_TABLE);
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        String table = "";

        switch (sUriMatcher.match(uri)) {
            case CONCRETE_TABLE:
                table = TestDatabaseHelper.Tables.CONCRETE;
                break;
            default:
                return 0;
        }

        int count = 0;
        SQLiteDatabase db = mDbHelper.getWritableDatabase();
        db.beginTransaction();

        try {
            count = db.delete(table, selection, selectionArgs);
            if(count>0) {
                db.delete(Tables.MEMORY, selection, selectionArgs);
            }
            db.setTransactionSuccessful();
        } finally {
            db.endTransaction();
        }

        if (count > 0) {
            notifyChange();
        }
        return count;
    }

    @Override
    public String getType(Uri uri) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
        String table = "";

        switch (sUriMatcher.match(uri)) {
            case CONCRETE_TABLE:
                table = TestDatabaseHelper.Tables.CONCRETE;
                break;
            default:
                return null;
        }

        SQLiteDatabase db = mDbHelper.getWritableDatabase();
        long id = -1;
        db.beginTransaction();


        try {
            id = db.insert(table, null, getValues(values, ConcreteColumns.UID));

            if (id > 0) {
                db.insert(TestDatabaseHelper.Tables.MEMORY, null,
                        getValues(values, MemoryColumns.CONCRETE_UID, MemoryColumns.PASSWORD));
            }
            db.setTransactionSuccessful();
        } finally {
            db.endTransaction();
        }
        Uri result = null;
        if (id > 0) {
            result = ContentUris.withAppendedId(uri, id);
            notifyChange();
        }
        return result;
    }

    private ContentValues getValues(ContentValues v, String...projection){
        ContentValues values = new ContentValues();
        for(String p: projection) {
            if(v.containsKey(p)) {
                values.put(p, v.get(p).toString());
            }
        }

        return values;
    }

    @Override
    public boolean onCreate() {
        mDbHelper = new TestDatabaseHelper(getContext());
        return false;
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs,
            String sortOrder) {
        StringBuilder sb = new StringBuilder();;

        switch (sUriMatcher.match(uri)) {
            case CONCRETE_TABLE:
                sb.append(TestDatabaseHelper.Tables.CONCRETE);
                break;
            default:
                return null;
        }

        Cursor cursor = null;
        SQLiteQueryBuilder qb = new SQLiteQueryBuilder();
        SQLiteDatabase db = mDbHelper.getReadableDatabase();

        if (isInProjection(projection, MemoryColumns.PASSWORD)) {
            sb.append(" LEFT OUTER JOIN " + Tables.MEMORY + " ON (" + Tables.CONCRETE + "."
                    + ConcreteColumns.UID + " = " + Tables.MEMORY + "."
                    + MemoryColumns.CONCRETE_UID + ")");
        }

        qb.setTables(sb.toString());
        qb.setProjectionMap(new HashMap<String, String>() {
            {
                put(MemoryColumns.PASSWORD, Tables.MEMORY + "." + MemoryColumns.PASSWORD);
                put(ConcreteColumns._ID, Tables.CONCRETE + "." + ConcreteColumns._ID);
                put(ConcreteColumns.UID, Tables.CONCRETE + "." + ConcreteColumns.UID);
            }
        });

        cursor = qb.query(db, projection, selection, selectionArgs, null, null, sortOrder);
        if (cursor != null) {
            cursor.setNotificationUri(getContext().getContentResolver(), uri);
        }

        return cursor;
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        int count = 0;
        SQLiteDatabase db = mDbHelper.getWritableDatabase();

        String table = "";

        switch (sUriMatcher.match(uri)) {
            case CONCRETE_TABLE:
                table = TestDatabaseHelper.Tables.CONCRETE;
                break;
            case MEMORY_TABLE:
                table = TestDatabaseHelper.Tables.MEMORY;
                break;
            default:
                return 0;
        }

        db.beginTransaction();
        try {
            count = db.update(table, values, selection, selectionArgs);
            db.setTransactionSuccessful();
        } finally {
            db.endTransaction();
        }

        if (count > 0) {
            notifyChange();
        }

        return count;
    }

    private boolean isInProjection(String[] projection, String column) {
        if (projection == null) {
            return true;
        }

        for (String test : projection) {
            if (column.equals(test)) {
                return true;
            }
        }
        return false;
    }

    private void notifyChange() {
        getContext().getContentResolver().notifyChange(CONTENT_URI, null, false);
    }
}
